@@ -1,28 +1,31 @@
 # Oraperl Emulation Interface for Perl 5 DBD::Oracle DBI
 #
-# Oraperl.pm
+# $Id: Oraperl.pm,v 1.39 1999/06/05 03:23:07 timbo Exp $
 #
 #   Copyright (c) 1994,1995 Tim Bunce
 #
-#   See the COPYRIGHT section in the Oracle.pm file for terms.
+#   You may distribute under the terms of either the GNU General Public
+#   License or the Artistic License, as specified in the Perl README file,
+#   with the exception that it cannot be placed on a CD-ROM or similar media
+#   for commercial distribution without the prior approval of the author.
 #
 # To use this interface use one of the following invocations:
 #
 #       use Oraperl;
 # or
-#       eval 'use Oraperl; 1;' || die $@ if $] >= 5;
+#       eval 'use Oraperl; 1' || die $@ if $] >= 5;
 #
 # The second form allows oraperl scripts to be used with
 # both oraperl and perl 5.
 
 package Oraperl;
 
-require 5.004;
+require 5.002;
 
-use DBI 1.22;
+use DBI 0.84;
 use Exporter;
 
-$VERSION = substr(q$Revision: 1.44 $, 10);
+$VERSION = substr(q$Revision: 1.39 $, 10);
 
 @ISA = qw(Exporter);
 
@@ -74,7 +77,7 @@ sub _warn {
     if ($_[0] =~ /^(Bad|Duplicate) free/) {
 	return unless $ENV{PERL_DBD_DUMP} eq 'dump';
 	print STDERR "Aborting with a core dump for diagnostics (PERL_DBD_DUMP)\n";
-	CORE::dump;
+	dump;
     }
     $prev_warn ? &$prev_warn(@_) : warn @_;
 }
@@ -89,9 +92,9 @@ sub ora_login {
     my($system_id, $name, $password) = @_;
     local($Oraperl::prev_warn) = $SIG{'__WARN__'} || 0; # must be local
     local($SIG{'__WARN__'}) = sub { _warn($Oraperl::prev_warn, @_) };
-    return DBI->connect("dbi:Oracle:$system_id", $name, $password, {
-	PrintError => 0, AutoCommit => 0
-    });
+    # we still use the old style connect call with an explicit driver
+    my $dbh = DBI->connect($system_id, $name, $password, 'Oracle');
+    return $dbh;
 }
 sub ora_logoff {
     my($dbh) = @_;
@@ -195,8 +198,11 @@ sub ora_version {
 #
 #   $ora_errno
 #   $ora_errstr
-*Oraperl::ora_errno  = \$DBI::err;
-*Oraperl::ora_errstr = \$DBI::errstr;
+
+# This is really internal knowledge but it saves using tie and
+# performance for ora_errno is very important.
+*Oraperl::ora_errno  = \$DBD::Oracle::err;
+*Oraperl::ora_errstr = \$DBD::Oracle::errstr;
 
 
 # -----------------------------------------------------------------
@@ -229,11 +235,11 @@ __END__
 
 =head1 NAME
 
-Oraperl - [DEPRECATED] Perl access to Oracle databases for old oraperl scripts
+Oraperl - Perl access to Oracle databases for old oraperl scripts
 
 =head1 SYNOPSIS
 
-  eval 'use Oraperl; 1;' || die $@ if $] >= 5;  # ADD THIS LINE TO OLD SCRIPTS
+  eval 'use Oraperl; 1' || die $@ if $] >= 5;  # ADD THIS LINE TO OLD SCRIPTS
 
   $lda = &ora_login($system_id, $name, $password)
   $csr = &ora_open($lda, $stmt [, $cache])
@@ -265,13 +271,6 @@ Oraperl - [DEPRECATED] Perl access to Oracle databases for old oraperl scripts
 
 Oraperl is an extension to Perl which allows access to Oracle databases.
 
-The original oraperl was a Perl 4 binary with Oracle OCI compiled into it.
-The Perl 5 Oraperl module described here is distributed with L<DBD::Oracle>
-(a database driver what operates within L<DBI>) and adds an extra layer over
-L<DBI> method calls.
-The Oraperl module should only be used to allow existing Perl 4 oraperl scripts
-to run with minimal changes; any new development should use L<DBI> directly.
-
 The functions which make up this extension are described in the
 following sections. All functions return a false or undefined (in the
 Perl sense) value to indicate failure.  You do not need to understand
@@ -290,12 +289,7 @@ Oraperl;>> in each file or package. If you need to make the scripts work
 with both the perl4 oraperl and perl5 you should add add the following
 text instead:
 
-  eval 'use Oraperl; 1;' || die $@ if $] >= 5;
-
-
-The use of I<Oraperl> is deprecated in favor of L<DBD::Oracle>,
-and will be removed from the I<DBD::Oracle> distribution as of 
-1.38.
+  eval 'use Oraperl; 1' || die $@ if $] >= 5;
 
 =head2 Principal Functions
 
@@ -834,7 +828,7 @@ It is quite possible, indeed probable, that some differences in
 behaviour will exist. These are probably confined to error handling.
 
 B<All> differences in behaviour which are not documented here should be
-reported to to dbi-users@perl.org.
+reported to Tim.Bunce@ig.co.uk B<and> CC'd to dbi-users@fugue.com.
 
 
 =head1 SEE ALSO
@@ -857,25 +851,16 @@ perl(1)
 
 =back
 
-=head1 AUTHOR
+=head1 AUTHORS
+
+Perl by Larry Wall <lwall@netlabs.com>.
+
+ORACLE by Oracle Corporation, California.
 
 Original Oraperl 2.4 code and documentation
 by Kevin Stock <kstock@auspex.fr>.
 
-DBI and Oraperl emulation using DBD::Oracle by Tim Bunce.
-
-=head1 MAINTAINER
-    
-As of DBD::Oracle release 1.17 in February 2006 The Pythian Group, Inc.
-(L<http://www.pythian.com>) are taking the lead in maintaining DBD::Oracle with
-my assistance and gratitude.
-
-=head1 COPYRIGHT
-
-Copyright (c) 1994-2006 Tim Bunce. Ireland.
-Copyright (c) 2006-2008 John Scoles (The Pythian Group). Canada.
-
-The DBD::Oracle module is free open source software; you can
-redistribute it and/or modify it under the same terms as Perl 5.
+DBI and Oraperl emulation using DBD::Oracle
+by <Tim.Bunce@ig.co.uk>
 
 =cut
